module Logger
  %w[ black red green yellow blue magenta cyan white ].each do |c|
    define_method c do |m|
      return m if options[:log]
      set_color m, c.to_sym
    end
  end

  def log message, level
    '%s [%s] %s -- %s' % [
      level,
      Time.now.iso8601(6),
      File::basename(caller[4]),
      message
    ]
  end

  def out
    begin
      @@log
    rescue NameError
      started = File.exist? options[:log]
      @@log   = File.open options[:log], 'a'
      @@log.puts '# Log started %s' % Time.now unless started
      @@log
    end
  end

  def stderr
    return out if options[:log]
    $stderr
  end

  def stdout
    return out if options[:log]
    $stdout
  end

  def info m
    if options[:debug] || options[:trace]
      stdout.puts green(log(m, 'INFO'))
    else
      stdout.puts green(m)
    end
  end

  def debug m
    stderr.puts blue(log(m, 'DEBUG')) if options[:debug]
  end

  def trace m
    stderr.puts magenta(log(m, 'TRACE')) if options[:trace]
  end
end